'''initialize'''
from .glitch import GlitchBeautifier
from .cartoonise import CartooniseBeautifier
from .cartoongan import CartoonGanBeautifier
from .oilpainting import OilpaintingBeautifier
from .beziercurve import BezierCurveBeautifier
from .photomosaic import PhotomosaicBeautifier
from .characterize import CharacterizeBeautifier
from .douyineffect import DouyinEffectBeautifier
from .noteprocessor import NoteprocessorBeautifier
from .pencildrawing import PencilDrawingBeautifier
from .cartoonizeface import CartoonizeFaceBeautifier
from .nostalgicstyle import NostalgicstyleBeautifier
from .photocorrection import PhotocorrectionBeautifier
from .fastneuralstyletransfer import FastNeuralStyleTransferBeautifier
from .geneticfitting import GeneticFittingCircleBeautifier, GeneticFittingPolygonBeautifier